#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


## Compute transonic potential flow around a RAE 2822
# Adrien Crovato
#
# Test the adaptive mesh refinement capability
#
# CAUTION
# This test is not stable. The adaptive mesh refinement procedure must be improved.

import dart.utils as floU
import dart.default as floD
import tbox.utils as tboxU
import fwk
from fwk.testing import *
import math

def main():
    # timer
    tms = fwk.Timers()
    tms['total'].start()

    # define flow variables
    alpha = 2*math.pi/180 # must be zero for this testfile
    M_inf = 0.72
    c_ref = 1
    dim = 2

    # define initial mesh parameters
    mName = 'rae2822'
    pars = {'xLgt' : 5, 'yLgt' : 5, 'msF' : 2.0, 'msTe' : 0.1, 'msLe' : 0.1}

    # AMR loop
    counter = 0
    Cls = []
    Cds = []
    Cms = []
    while True:
        # mesh the geometry
        tms['msh'].start()
        msh, gmshWriter = floD.mesh(dim, f'models/{mName}.geo', pars, ['field', 'airfoil', 'downstream'])
        tms['msh'].stop()

        # set the problem and add fluid, initial/boundary conditions
        tms['pre'].start()
        pbl, _, _, bnd, _ = floD.problem(msh, dim, alpha, 0., M_inf, c_ref, c_ref, 0.25, 0., 0., 'airfoil')
        tms['pre'].stop()

        # solve problem
        tms['solver'].start()
        solver = floD.newton(pbl)
        solver.run()
        solver.save(gmshWriter)
        stop = solver.refine(0.005, gmshWriter)
        tms['solver'].stop()

        # extract data
        Cls.append(solver.Cl)
        Cds.append(solver.Cd)
        Cms.append(solver.Cm)
        Cp = floU.extract(bnd.groups[0].tag.elems, solver.Cp)
        tboxU.write(Cp, f'Cp_airfoil_{counter}.dat', '%1.4e', ',', 'x, y, cp', '')

        # stop criterion
        if counter > 10 or stop:
            break
        else:
            counter += 1
            pars['-bgm'] = f'{mName}_size.pos'
            pars['counter'] = counter # pass the counter to force remesh

    # display results
    print('{0:>8s} {1:>8s} {2:>8s} {3:>8s} {4:>8s} {5:>8s}'.format('run', 'Mach', 'AoA', 'Cl', 'Cd', 'Cm'))
    i = 0
    while i < len(Cls):
        print('{0:8d} {1:8.3f} {2:8.1f} {3:8.4f} {4:8.4f} {5:8.4f}'.format(i, M_inf, alpha*180/math.pi, Cls[i], Cds[i], Cms[i]))
        i = i+1
    print('\n')

    # display timers
    tms['total'].stop()
    print('CPU statistics')
    print(tms)

    # visualize solution and plot results
    floD.initViewer(pbl)
    floD.plot(Cp[:,0], Cp[:,-1], {'xlabel': 'x', 'ylabel': 'Cp', 'title': 'Cl = {0:.{3}f}, Cd = {1:.{3}f}, Cm = {2:.{3}f}'.format(solver.Cl, solver.Cd, solver.Cm, 4), 'invert': True})

    # check results
    tests = CTests()
    tests.add(CTest('iteration count', solver.nIt, 15, 3, forceabs=True))
    tests.add(CTest('refinement count', counter, 4, 1, forceabs=True))
    tests.add(CTest('Cl', solver.Cl, 0.85, 1e-2))
    tests.add(CTest('Cd', solver.Cd, 0.0025, 1e-2))
    tests.run()

    # eof
    print('')

if __name__ == "__main__":
    main()
